@@ -1,7 +1,7 @@
 #
 # backend.py: interface for backends
 #
-# Copyright (C) 2008 Canonical Ltd.
+# Copyright (C) 2008-2009 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -40,6 +40,12 @@ class UFWBackend:
                       'apps': os.path.join(config_dir, 'ufw/applications.d') }
         self.files.update(extra_files)
 
+        self.loglevels = {'off':       0,
+                          'low':     100,
+                          'medium':  200,
+                          'high':    300,
+                          'full':    400 }
+
         self.do_checks = True
         try:
             self._do_checks()
@@ -49,6 +55,7 @@ class UFWBackend:
             raise
 
         self.profiles = ufw.applications.get_profiles(self.files['apps'])
+        self.iptables_version = ufw.util.get_iptables_version()
 
     def _is_enabled(self):
         if self.defaults.has_key('enabled') and \
@@ -100,24 +107,20 @@ class UFWBackend:
         warned_group_write = {}
         warned_owner = {}
 
-        pat = re.compile(r'^\.')
-
         profiles = []
         if not os.path.isdir(self.files['apps']):
             warn_msg = _("'%s' does not exist") % (self.files['apps'])
             warn(warn_msg)
         else:
+            pat = re.compile(r'^\.')
             for profile in os.listdir(self.files['apps']):
-                profiles.append(os.path.join(self.files['apps'], profile))
+                if not pat.search(profile):
+                    profiles.append(os.path.join(self.files['apps'], profile))
 
         for path in self.files.values() + [ os.path.abspath(sys.argv[0]) ] + \
                 profiles:
             while True:
                 debug("Checking " + path)
-                if pat.search(os.path.basename(path)):
-                    err_msg = _("found hidden directory in path: %s") % (path)
-                    raise UFWError(err_msg)
-
                 if path == self.files['apps'] and \
                            not os.path.isdir(self.files['apps']):
                     break
@@ -187,13 +190,19 @@ class UFWBackend:
             raise
         fd = fns['tmp']
 
+        found = False
         pat = re.compile(r'^' + opt + '=')
         for line in fns['orig']:
             if pat.search(line):
                 os.write(fd, opt + "=" + value + "\n")
+                found = True
             else:
                 os.write(fd, line)
 
+        # Add the entry if not found
+        if not found:
+            os.write(fd, opt + "=" + value + "\n")
+
         ufw.util.close_files(fns)
 
         # Now that the files are written out, update value in memory
@@ -210,6 +219,10 @@ class UFWBackend:
                 self.set_default(self.files['defaults'], \
                                             "DEFAULT_APPLICATION_POLICY", \
                                             "\"DROP\"")
+            elif policy == "reject":
+                self.set_default(self.files['defaults'], \
+                                            "DEFAULT_APPLICATION_POLICY", \
+                                            "\"REJECT\"")
             elif policy == "skip":
                 self.set_default(self.files['defaults'], \
                                             "DEFAULT_APPLICATION_POLICY", \
@@ -218,7 +231,7 @@ class UFWBackend:
                 err_msg = _("Unsupported policy '%s'") % (policy)
                 raise UFWError(err_msg)
 
-        rstr = _("Default application policy changed to '%s'\n") % (policy)
+        rstr = _("Default application policy changed to '%s'") % (policy)
 
         return rstr
 
@@ -370,22 +383,168 @@ class UFWBackend:
 
         return (rstr, updated_profile)
 
-    # API overrides
+    def find_application_name(self, str):
+        '''Find the application profile name for str'''
+        if self.profiles.has_key(str):
+            return str
+
+        match = ""
+        matches = 0
+        for n in self.profiles.keys():
+            if n.lower() == str.lower():
+                match = n
+                matches += 1
+
+        debug_msg = "'%d' matches for '%s'" % (matches, str)
+        debug(debug_msg)
+        if matches == 1:
+            return match
+        elif matches > 1:
+            err_msg = _("Found multiple matches for '%s'. Please use exact profile name") % (str)
+        err_msg = _("Could not find a profile matching '%s'") % (str)
+        raise UFWError(err_msg)
+
+    def find_other_position(self, position, v6):
+	'''Return the absolute position in the other list of the rule with the
+	   user position of the given list. For example, find_other_position(4,
+	   True) will return the absolute position of the rule in the ipv4 list
+           matching the user specified '4' rule in the ipv6 list.
+        '''
+        # Invalid search (v6 rule with too low position)
+        if v6 and position > len(self.rules6):
+            raise ValueError()
+
+        # Invalid search (v4 rule with too high position)
+        if not v6 and position > len(self.rules):
+            raise ValueError()
+
+        if position < 1:
+            raise ValueError()
+
+        rules = []
+        if v6:
+            rules = self.rules6
+        else:
+            rules = self.rules
+
+        # self.rules[6] is a list of tuples. Some application rules have
+        # multiple tuples but the user specifies by ufw rule, not application
+        # tuple, so we need to find how many tuples there are leading up to
+        # the specified position, which we can then use as an offset for
+        # getting the proper match_rule.
+        app_rules = {}
+        tuple_offset = 0
+        for i, r in enumerate(rules):
+            if i >= position:
+                break
+            tuple = ""
+            if r.dapp != "" or r.sapp != "":
+                tuple = r.get_app_tuple()
+
+                if app_rules.has_key(tuple):
+                    tuple_offset += 1
+                else:
+                    app_rules[tuple] = True
+
+        rules = []
+        if v6:
+            rules = self.rules
+            match_rule = self.rules6[position - 1 + tuple_offset].dup_rule()
+            match_rule.set_v6(False)
+        else:
+            rules = self.rules6
+            match_rule = self.rules[position - 1 + tuple_offset].dup_rule()
+            match_rule.set_v6(True)
+
+        count = 1
+        for r in rules:
+            if UFWRule.match(r, match_rule) == 0:
+                return count
+            count += 1
+
+        return 0
+
     def get_loglevel(self):
-        raise UFWError("UFWBackend.get_loglevel: need to override")
+        '''Gets current log level of firewall'''
+        level = 0
+        rstr = _("Logging: ")
+        if not self.defaults.has_key('loglevel') or \
+           self.defaults['loglevel'] not in self.loglevels.keys():
+            level = -1
+            rstr += _("unknown")
+        else:
+            level = self.loglevels[self.defaults['loglevel']]
+            if level == 0:
+                rstr += "off"
+            else:
+                rstr += "on (%s)" % (self.defaults['loglevel'])
+        return (level, rstr)
 
     def set_loglevel(self, level):
-        raise UFWError("UFWBackend.set_loglevel: need to override")
+        '''Sets log level of firewall'''
+        if level not in self.loglevels.keys() + ['on']:
+            err_msg = _("Invalid log level '%s'") % (level)
+            raise UFWError(err_msg)
 
+        new_level = level
+        if level == "on":
+           if not self.defaults.has_key('loglevel') or \
+              self.defaults['loglevel'] == "off":
+               new_level = "low"
+           else:
+               new_level = self.defaults['loglevel']
+
+        self.set_default(self.files['conf'], "LOGLEVEL", new_level)
+        try:
+            self.update_logging(new_level)
+        except:
+            raise
+
+        if new_level == "off":
+            return _("Logging disabled")
+        else:
+            return _("Logging enabled")
+
+    def get_rules_count(self, v6):
+        '''Return number of ufw rules (not iptables rules)'''
+        rules = []
+        if v6:
+            rules = self.rules6
+        else:
+            rules = self.rules
+
+        count = 0
+        app_rules = {}
+        for r in rules:
+            tuple = ""
+            if r.dapp != "" or r.sapp != "":
+                tuple = r.get_app_tuple()
+
+                if app_rules.has_key(tuple):
+                    debug("Skipping found tuple '%s'" % (tuple))
+                    continue
+                else:
+                    app_rules[tuple] = True
+            count += 1
+
+        return count
+
+    # API overrides
     def get_default_policy(self):
         raise UFWError("UFWBackend.get_default_policy: need to override")
 
     def set_default_policy(self, policy):
         raise UFWError("UFWBackend.set_default_policy: need to override")
 
-    def get_status(self):
+    def get_running_raw(self):
+        raise UFWError("UFWBackend.get_running_raw: need to override")
+
+    def get_status(self, verbose, show_count):
         raise UFWError("UFWBackend.get_status: need to override")
 
+    def get_status_as_list(self):
+        raise UFWError("UFWBackend.get_status_as_list: need to override")
+
     def set_rule(self, rule, allow_reload):
         raise UFWError("UFWBackend.set_rule: need to override")
 
@@ -399,3 +558,6 @@ class UFWBackend:
         raise UFWError("UFWBackend.get_app_rules_from_system: need to " + \
                        "override")
 
+    def update_logging(self, level):
+        raise UFWError("UFWBackend.update_logging: need to override")
+
