@@ -1,7 +1,7 @@
 #
 # backend_iptables.py: iptables backend for ufw
 #
-# Copyright (C) 2008 Canonical Ltd.
+# Copyright (C) 2008-2009 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -21,8 +21,8 @@ import re
 import sys
 import tempfile
 
-from ufw.common import UFWError, UFWRule, config_dir, state_dir
-from ufw.util import warn, debug, cmd, cmd_pipe
+from ufw.common import UFWError, UFWRule, config_dir, state_dir, prefix_dir
+from ufw.util import warn, debug, msg, cmd, cmd_pipe
 import ufw.backend
 
 
@@ -37,43 +37,37 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
         files['rules6'] = os.path.join(state_dir, 'user6.rules')
         files['before6_rules'] = os.path.join(config_dir, 'ufw/before6.rules')
         files['after6_rules'] = os.path.join(config_dir, 'ufw/after6.rules')
-        files['init'] = os.path.join(config_dir, 'init.d/ufw')
+        files['init'] = os.path.join(prefix_dir, 'share/ufw/ufw-init')
 
         ufw.backend.UFWBackend.__init__(self, "iptables", d, files)
 
-    def get_loglevel(self):
-        '''Gets current log level of firewall'''
-        level = 1
-        rstr = _("Logging: on")
-        for f in [self.files['rules'], self.files['rules6'], \
-                  self.files['before_rules'], self.files['before6_rules'], \
-                  self.files['after_rules'], self.files['after6_rules']]:
-            try:
-                orig = ufw.util.open_file_read(f)
-            except Exception:
-                err_msg = _("Couldn't open '%s' for reading") % (f)
-                raise UFWError(err_msg)
+        self.chains = {'before': [], 'user': [], 'after': [], 'misc': []}
+        for ver in ['4', '6']:
+            chain_prefix = "ufw"
+            if ver == "6":
+                if self.use_ipv6():
+                    chain_prefix += ver
+                elif ver == "6":
+                    continue
 
-            for line in orig:
-                # If find one occurence of the comment_str, we know the user
-                # ran "logging off"
-                if self.comment_str in line:
-                    rstr = _("Logging: off")
-                    level = 0
-                    orig.close()
-                    break
+            for loc in ['before', 'user', 'after']:
+                for target in ['input', 'output', 'forward']:
+                   chain = "%s-%s-logging-%s" % (chain_prefix, loc, target)
+                   self.chains[loc].append(chain)
+            self.chains['misc'].append(chain_prefix + "-logging-deny")
+            self.chains['misc'].append(chain_prefix + "-logging-allow")
 
-            orig.close()
-
-        return (level, rstr)
-
-    def get_default_policy(self):
+    def get_default_policy(self, primary="input"):
         '''Get current policy'''
-        rstr = _("Default:")
-        if self.defaults['default_input_policy'] == "accept":
-            rstr += " allow"
+        policy = "default_" + primary + "_policy"
+
+        rstr = ""
+        if self.defaults[policy] == "accept":
+            rstr = "allow"
+        elif self.defaults[policy] == "reject":
+            rstr = "reject"
         else:
-            rstr += " deny"
+            rstr = "deny"
 
         return rstr
 
@@ -84,6 +78,8 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
             rstr += " allow"
         elif self.defaults['default_application_policy'] == "drop":
             rstr += " deny"
+        elif self.defaults['default_application_policy'] == "reject":
+            rstr += " reject"
         else:
             rstr += " skip"
 
@@ -92,7 +88,7 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
     def set_default_policy(self, policy):
         '''Sets default policy of firewall'''
         if not self.dryrun:
-            if policy != "allow" and policy != "deny":
+            if policy != "allow" and policy != "deny" and policy != "reject":
                 err_msg = _("Unsupported policy '%s'") % (policy)
                 raise UFWError(err_msg)
 
@@ -104,6 +100,12 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
                                             "\"ACCEPT\"")
                 old_log_str = 'UFW BLOCK'
                 new_log_str = 'UFW ALLOW'
+            elif policy == "reject":
+                self.set_default(self.files['defaults'], \
+                                            "DEFAULT_INPUT_POLICY", \
+                                            "\"REJECT\"")
+                old_log_str = 'UFW ALLOW'
+                new_log_str = 'UFW BLOCK'
             else:
                 self.set_default(self.files['defaults'], \
                                             "DEFAULT_INPUT_POLICY", \
@@ -133,45 +135,33 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
 
         return rstr
 
-    def set_loglevel(self, level):
-        '''Sets log level of firewall'''
-        for f in [self.files['rules'], self.files['rules6'], \
-                  self.files['before_rules'], self.files['before6_rules'], \
-                  self.files['after_rules'], self.files['after6_rules']]:
-            try:
-                fns = ufw.util.open_files(f)
-            except Exception:
-                raise
-            fd = fns['tmp']
+    def get_running_raw(self):
+        '''Show current running status of firewall'''
+        if self.dryrun:
+            out = "> " + _("Checking raw iptables\n")
+            out += "> " + _("Checking raw ip6tables\n")
+            return out
 
-            pat = re.compile(r'^-.*\sLOG\s')
-            if level == "on":
-                pat = re.compile(r'^#.*\sLOG\s')
+        err_msg = _("problem running")
 
-            if not self.dryrun:
-                for line in fns['orig']:
-                    if pat.search(line):
-                        if level == "off":
-                            os.write(fd, self.comment_str + ' ' + line)
-                        else:
-                            pat_comment = re.compile(r"^" + \
-                                                     self.comment_str + "\s*")
-                            os.write(fd, pat_comment.sub('', line))
-                    else:
-                        os.write(fd, line)
+        out = "IPV4:\n"
+        for table in ['filter', 'nat', 'mangle', 'raw']:
+            (rc, tmp) = cmd(['iptables', '-L', '-n', '-v', '-x', '-t', table])
+            out += tmp
+            if rc != 0:
+                raise UFWError(out)
 
-            if self.dryrun:
-                ufw.util.close_files(fns, False)
-            else:
-                ufw.util.close_files(fns)
+        out += "\n\nIPV6:\n"
+        for table in ['filter', 'mangle', 'raw']:
+            (rc, tmp) = cmd(['ip6tables', '-L', '-n', '-v', '-x', '-t', table])
+            out += tmp
+            if rc != 0:
+                raise UFWError(out)
 
-        if level == "off":
-            return _("Logging disabled")
-        else:
-            return _("Logging enabled")
+        return out
 
-    def get_status(self, verbose=False):
-        '''Show current status of firewall'''
+    def get_status(self, verbose=False, show_count=False):
+        '''Show ufw managed rules'''
         out = ""
         out6 = ""
         if self.dryrun:
@@ -183,51 +173,20 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
         # Is the firewall loaded at all?
         (rc, out) = cmd(['iptables', '-L', 'ufw-user-input', '-n'])
         if rc != 0:
-            return _("Status: not loaded")
+            return _("Status: inactive")
 
         err_msg = _("problem running")
-
-        # Get the output of iptables for parsing
-        (rc, out) = cmd(['iptables', '-L', '-n'])
-        if rc != 0:
-            raise UFWError(err_msg + " iptables")
-
         if self.use_ipv6():
             (rc, out6) = cmd(['ip6tables', '-L', 'ufw6-user-input', '-n'])
             if rc != 0:
                 raise UFWError(err_msg + " ip6tables")
-            if out6 == "":
-                return out6
 
         if out == "" and out6 == "":
-            return _("Status: loaded")
+            return _("Status: active")
 
         str = ""
-        rules = []
-        pat_chain = re.compile(r'^Chain ')
-        pat_target = re.compile(r'^target')
-        for type in ["v4", "v6"]:
-            pat_ufw = re.compile(r'^Chain ufw-user-input')
-            if type == "v6":
-                pat_ufw = re.compile(r'^Chain ufw6-user-input')
-            lines = out
-            if type == "v6":
-                lines = out6
-            in_ufw_input = False
-            for line in lines.split('\n'):
-                if pat_ufw.search(line):
-                    in_ufw_input = True
-                    continue
-                elif pat_chain.search(line):
-                    in_ufw_input = False
-                    continue
-                elif pat_target.search(line):
-                    pass
-                elif in_ufw_input:
-                    r = self._parse_iptables_status(line, type)
-                    if r is not None:
-                        rules.append(r)
-
+        rules = self.rules + self.rules6
+        count = 1
         app_rules = {}
         for r in rules:
             location = {}
@@ -308,93 +267,70 @@ class UFWBackendIptables(ufw.backend.UFWBackend):
                            r.dport == r.sport:
                             location[loc] += "/" + r.protocol
 
-            str += "%-26s %-8s%s\n" % (location['dst'], r.action.upper(), \
-                    location['src'])
+            if show_count:
+                str += "[%2d] " % (count)
+            
+            log_str = ""
+            if r.logtype:
+                log_str = " (%s)" % (r.logtype.lower())
+            str += "%-26s %-8s%s%s\n" % (location['dst'], r.action.upper(), \
+                    location['src'], log_str)
+            count += 1
 
         if str != "":
-            header = "\n\n%-26s %-8s%s\n" % (_("To"), _("Action"), _("From"))
+            header = "\n\n"
+            if show_count:
+                header += "     "
+            header += "%-26s %-8s%s\n" % (_("To"), _("Action"), _("From"))
+            if show_count:
+                header += "     "
             header += "%-26s %-8s%s\n" % (_("--"), _("------"), _("----"))
             str = header + str
 
         if verbose:
             (level, logging_str) = self.get_loglevel()
-            policy_str = self.get_default_policy()
+            policy_str = _("Default: %s") % (self.get_default_policy())
             app_policy_str = self.get_default_application_policy()
-            return _("Status: loaded\n%s\n%s\n%s%s") % \
+            return _("Status: active\n%s\n%s\n%s%s") % \
                                                     (logging_str, policy_str, \
                                                      app_policy_str, str)
         else:
-            return _("Status: loaded%s") % (str)
+            return _("Status: active%s") % (str)
 
     def stop_firewall(self):
         '''Stops the firewall'''
-        openconf = '''*filter
-:INPUT ACCEPT [0:0]
-:FORWARD ACCEPT [0:0]
-:OUTPUT ACCEPT [0:0]
-COMMIT
-'''
-        if self.dryrun:
-            print "> iptables -F"
-            print "> iptables -X"
-            print "> echo\n" + openconf + "> | iptables-restore"
-            if self.use_ipv6():
-                print "> ip6tables -F"
-                print "> ip6tables -X"
-                print "> echo\n" + openconf + "> | ip6tables-restore"
-            return
-
-        try:
-            tmp = tempfile.NamedTemporaryFile()
-        except Exception:
-            raise
-        tmp.write(openconf)
-        tmp.flush()
-
         err_msg = _("problem running")
-
-        # Flush the firewall
-        (rc, out) = cmd(['iptables', '-F'])
-        if rc != 0:
-            raise UFWError(err_msg + " iptables 'flush'")
-
-        # Remove user chains
-        (rc, out) = cmd(['iptables', '-X'])
-        if rc != 0:
-            raise UFWError(err_msg + " iptables 'delete'")
-
-        # Set default open
-        (rc, out) = cmd_pipe(['cat', tmp.name], ['iptables-restore'])
-        if rc != 0:
-            raise UFWError(err_msg + " iptables")
-
-        if self.use_ipv6():
-            # Flush the firewall
-            (rc, out) = cmd(['ip6tables', '-F'])
+        if self.dryrun:
+            msg("> " + _("running ufw-init"))
+        else:
+            (rc, out) = cmd([self.files['init'], 'force-stop'])
             if rc != 0:
-                raise UFWError(err_msg + " ip6tables 'flush'")
-
-            # Remove user chains
-            (rc, out) = cmd(['ip6tables', '-X'])
-            if rc != 0:
-                raise UFWError(err_msg + " ip6tables 'delete'")
-
-            # Set default open
-            (rc, out) = cmd_pipe(['cat', tmp.name], ['ip6tables-restore'])
-            if rc != 0:
-                raise UFWError(err_msg + " ip6tables")
-
-        tmp.close()
+                raise UFWError(err_msg + " ufw-init")
 
     def start_firewall(self):
         '''Starts the firewall'''
         err_msg = _("problem running")
         if self.dryrun:
-            print "> " + _("running initscript")
+            msg("> " + _("running ufw-init"))
         else:
             (rc, out) = cmd([self.files['init'], 'start'])
             if rc != 0:
-                raise UFWError(err_msg + " init script")
+                raise UFWError(err_msg + " ufw-init")
+
+            if not self.defaults.has_key('loglevel') or \
+               self.defaults['loglevel'] not in self.loglevels.keys():
+                # Add the loglevel if not valid
+                try:
+                    self.set_loglevel("low")
+                except:
+                    err_msg = _("Could not set LOGLEVEL")
+                    raise UFWError(err_msg)
+            else:
+                try:
+                    self.update_logging(self.defaults['loglevel'])
+                except:
+                    err_msg = _("Could not load logging rules")
+                    raise UFWError(err_msg)
 
     def _need_reload(self, v6):
         '''Check if all chains exist'''
@@ -422,10 +358,19 @@ COMMIT
         '''Reload firewall rules file'''
         err_msg = _("problem running")
         if self.dryrun:
-            print "> | iptables-restore"
+            msg("> | iptables-restore")
             if self.use_ipv6():
-                print "> | ip6tables-restore"
+                msg("> | ip6tables-restore")
         elif self._is_enabled():
+            # first flush the user logging chains
+            try:
+                for c in self.chains['user']:
+                    self._chain_cmd(c, ['-F', c])
+                    self._chain_cmd(c, ['-Z', c])
+            except:
+                raise UFWError(err_msg)
+
+            # then restore the system rules
             (rc, out) = cmd_pipe(['cat', self.files['rules']], \
                                  ['iptables-restore', '-n'])
             if rc != 0:
@@ -441,17 +386,55 @@ COMMIT
         '''Return list of iptables rules appropriate for sending'''
         snippets = []
 
+        # adjust reject and protocol 'all'
         pat_proto = re.compile(r'-p all ')
         pat_port = re.compile(r'port ')
+        pat_reject = re.compile(r'-j (REJECT(_log(-all)?)?)')
         if pat_proto.search(frule):
             if pat_port.search(frule):
-                snippets.append(pat_proto.sub('-p tcp ', frule))
+                if pat_reject.search(frule):
+                    snippets.append(pat_proto.sub('-p tcp ', \
+                        pat_reject.sub(r'-j \1 --reject-with tcp-reset ', \
+                        frule)))
+                else:
+                    snippets.append(pat_proto.sub('-p tcp ', frule))
                 snippets.append(pat_proto.sub('-p udp ', frule))
             else:
                 snippets.append(pat_proto.sub('', frule))
         else:
             snippets.append(frule)
 
+        # adjust for logging rules
+        pat_log = re.compile(r'(.*)-j ([A-Z]+)_log(-all)?(.*)')
+        pat_logall = re.compile(r'-j [A-Z]+_log-all')
+        pat_chain = re.compile(r'(-A|-D) ([a-zA-Z0-9\-]+)')
+        limit_args = '-m limit --limit 3/min --limit-burst 10'
+        for i, s in enumerate(snippets):
+            if pat_log.search(s):
+                policy = pat_log.sub(r'\2', s).strip()
+                if policy.lower() == "accept":
+                    policy = "ALLOW"
+                elif policy.lower() == "limit":
+                    policy = "LIMIT"
+                else:
+                    policy = "BLOCK"
+
+                lstr = '%s -j LOG --log-prefix "[UFW %s] "' % (limit_args, \
+                       policy)
+                if not pat_logall.search(s):
+                    lstr = '-m state --state NEW ' + lstr
+                snippets[i] = pat_log.sub(r'\1-j \2\4', s)
+                snippets.insert(i, pat_log.sub(r'\1-j ' + prefix + \
+                                               '-user-logging-input', s))
+                snippets.insert(i, pat_chain.sub(r'\1 ' + prefix + \
+                                                 '-user-logging-input', 
+                                                 pat_log.sub(r'\1-j RETURN', \
+                                                 s)))
+                snippets.insert(i, pat_chain.sub(r'\1 ' + prefix + \
+                                                 '-user-logging-input', 
+                                                 pat_log.sub(r'\1' + lstr, s)))
+
+        # adjust for limit
         pat_limit = re.compile(r' -j LIMIT')
         for i, s in enumerate(snippets):
             if pat_limit.search(s):
@@ -467,107 +450,23 @@ COMMIT
 
         return snippets
 
-    def _parse_iptables_status(self, line, type):
-        '''Parses a line from iptables -L -n'''
-        fields = line.split()
+    def _get_lists_from_formatted(self, frule, prefix):
+        '''Return list of iptables rules appropriate for sending as arguments
+           to cmd()
+        '''
+        snippets = []
+        str_snippets = self._get_rules_from_formatted(frule, prefix)
 
-        if type == "v6":
-            # ip6tables hack since its opt field is blank (unlike iptables)
-            fields.insert(2, '--')
+        # split the string such that the log prefix can contain spaces
+        pat = re.compile(r'(.*) --log-prefix (".* ")(.*)')
+        for i, s in enumerate(str_snippets):
+            snippets.append(pat.sub(r'\1', s).split())
+            if pat.match(s):
+                snippets[i].append("--log-prefix")
+                snippets[i].append(pat.sub(r'\2', s).replace('"', ''))
+                snippets[i] += pat.sub(r'\3', s).split()
 
-        if len(fields) < 5:
-            dbg_msg = _("Couldn't parse line '%s'") % (line)
-            debug(dbg_msg)
-            return None
-
-        rule = UFWRule("ACCEPT", "any", "any")
-        if fields[0] == 'ACCEPT':
-            rule.set_action('allow')
-        elif fields[0] == 'DROP':
-            rule.set_action('deny')
-        elif fields[0] == "ufw-user-limit-accept":
-            rule.set_action('limit')
-        else:
-            # RETURN and LOG are valid, but we skip them
-            return None
-
-        if fields[1] == 'tcp' or fields[1] == 'udp':
-            rule.set_protocol(fields[1])
-        elif fields[1] == "0" or fields[1] == "all":
-            rule.set_protocol('any')
-        else:
-            rule.set_protocol('UNKNOWN')
-
-        if type == "v6":
-            # ip6tables hack since it doesn't have a space between the
-            # destination address and the protocol on a large destination
-            # address (see Debian bug #464244).
-            mashed = fields[4][(len(fields[4]) - 3):]
-            if mashed == 'tcp' or mashed == 'udp':
-                fields.insert(5, mashed)
-                fields[4] = fields[4][:(len(fields[4]) - 3)]
-
-        try:
-            rule.set_src(fields[3])
-            rule.set_dst(fields[4])
-        except Exception:
-            warn_msg = _("Couldn't parse line '%s'") % (line)
-            warn(warn_msg)
-            return None
-
-        if len(fields) >= 7:
-            if re.match('dpt', fields[6]):
-                rule.set_port(fields[6][4:], "dst")
-            elif re.match('spt', fields[6]):
-                rule.set_port(fields[6][4:], "src")
-
-        if len(fields) >= 8:
-            if re.match('dpt', fields[7]):
-                rule.set_port(fields[7][4:], "dst")
-            elif re.match('spt', fields[7]):
-                rule.set_port(fields[7][4:], "src")
-            elif re.match('multiport', fields[5]):
-                if fields[6] == "dports":
-                    rule.set_port(fields[7], "dst")
-                elif fields[6] == "sports":
-                    rule.set_port(fields[7], "src")
-                if len(fields) >= 11:
-                    if fields[9] == "dports":
-                        rule.set_port(fields[10], "dst")
-                    elif fields[9] == "sports":
-                        rule.set_port(fields[10], "src")
-
-        has_comment = False
-        comments = []
-        try:
-            findex = fields.index('/*')
-            lindex = fields.index('*/')
-            has_comment = True
-        except Exception:
-            has_comment = False
-
-        if has_comment:
-            if findex + 1 >= lindex:
-                # Empty comment
-                has_comment = False
-            else:
-                comments = ' '.join(fields[findex+1:lindex]).strip("'").split()
-
-            if len(comments) > 0:
-                pat_space = re.compile('%20')
-                for app in comments[0].split(","):
-                    tmp = pat_space.sub(' ', app)
-                    if tmp.startswith('dapp_'):
-                        rule.dapp = tmp[5:]
-                    if tmp.startswith('sapp_'):
-                        rule.sapp = tmp[5:]
-
-        if type == "v6":
-            rule.set_v6(True)
-        else:
-            rule.set_v6(False)
-
-        return rule
+        return snippets
 
     def _read_rules(self):
         '''Read in rules that were added by ufw.'''
@@ -656,11 +555,13 @@ COMMIT
 
         # Write rules
         for r in rules:
-            rule_str = "-A " + chain_prefix + "-user-input " + \
-                       r.format_rule() + "\n"
+            action = r.action
+            if r.logtype != "":
+                action += "_" + r.logtype
+
             if r.dapp == "" and r.sapp == "":
                 os.write(fd, "\n### tuple ### %s %s %s %s %s %s\n" % \
-                     (r.action, r.protocol, r.dport, r.dst, r.sport, r.src))
+                     (action, r.protocol, r.dport, r.dst, r.sport, r.src))
             else:
                 pat_space = re.compile(' ')
                 dapp = "-"
@@ -670,23 +571,22 @@ COMMIT
                 if r.sapp:
                     sapp = pat_space.sub('%20', r.sapp)
                 os.write(fd, "\n### tuple ### %s %s %s %s %s %s %s %s\n" \
-                     % (r.action, r.protocol, r.dport, r.dst, r.sport, r.src, \
-                      dapp, sapp))
+                     % (action, r.protocol, r.dport, r.dst, r.sport, r.src, \
+                        dapp, sapp))
 
+            rule_str = "-A " + chain_prefix + "-user-input " + \
+                       r.format_rule() + "\n"
             for s in self._get_rules_from_formatted(rule_str, chain_prefix):
                 os.write(fd, s)
 
         # Write footer
         os.write(fd, "\n### END RULES ###\n")
-        os.write(fd, "-A " + chain_prefix + "-user-input -j RETURN\n")
-        os.write(fd, "-A " + chain_prefix + "-user-output -j RETURN\n")
-        os.write(fd, "-A " + chain_prefix + "-user-forward -j RETURN\n")
 
         if chain_prefix == "ufw":
             # Rate limiting only supported with IPv4
             os.write(fd, "-A " + chain_prefix + "-user-limit -m limit " + \
                          "--limit 3/minute -j LOG --log-prefix " + \
-                         "\"[UFW LIMIT]: \"\n")
+                         "\"[UFW LIMIT BLOCK] \"\n")
             os.write(fd, "-A " + chain_prefix + "-user-limit -j REJECT\n")
             os.write(fd, "-A " + chain_prefix + "-user-limit-accept -j ACCEPT\n")
 
@@ -701,6 +601,7 @@ COMMIT
         '''Updates firewall with rule by:
         * appending the rule to the chain if new rule and firewall enabled
         * deleting the rule from the chain if found and firewall enabled
+        * inserting the rule if possible and firewall enabled
         * updating user rules file
         * reloading the user rules file if rule is modified
         '''
@@ -724,29 +625,69 @@ COMMIT
         delete = False
 
         rules = self.rules
+        position = rule.position
         if rule.v6:
+            if self.iptables_version < "1.4" and (rule.dapp != "" or rule.sapp != ""):
+                return _("Skipping IPv6 application rule. Need at least iptables 1.4")
             rules = self.rules6
 
+        # bail if we have a bad position
+        if position < 0 or position > len(rules):
+            err_msg = _("Invalid position '%d'") % (position)
+            raise UFWError(err_msg)
+
+        if position > 0 and rule.remove:
+            err_msg = _("Cannot specify insert and delete")
+            raise UFWError(err_msg)
+        if position > len(rules):
+            err_msg = _("Cannot insert rule at position '%d'") % position
+            raise UFWError(err_msg)
+
         # First construct the new rules list
         try:
             rule.normalize()
         except Exception:
             raise
 
+        count = 1
+        inserted = False
+        matches = 0
+        last = ('', '', '', '')
         for r in rules:
             try:
                 r.normalize()
             except Exception:
                 raise
 
+            current = (r.dst, r.src, r.dapp, r.sapp)
+            if count == position:
+                # insert the rule if:
+                # 1. the last rule was not an application rule
+                # 2. the current rule is not an application rule
+                # 3. the last application rule is different than the current
+                #    while the new rule is different than the current one
+                if (last[2] == '' and last[3] == '' and count > 1) or \
+                   (current[2] == '' and current[3] == '') or \
+                   last != current:
+                    inserted = True
+                    newrules.append(rule)
+                    last = ('', '', '', '')
+                else:
+                    position += 1
+            last = current
+            count += 1
+
             ret = UFWRule.match(r, rule)
-            if ret == 0 and not found:
+            if ret < 1:
+                matches += 1
+
+            if ret == 0 and not found and not inserted:
                 # If find the rule, add it if it's not to be removed, otherwise
                 # skip it.
                 found = True
                 if not rule.remove:
                     newrules.append(rule)
-            elif ret < 0 and not rule.remove:
+            elif ret < 0 and not rule.remove and not inserted:
                 # If only the action is different, replace the rule if it's not
                 # to be removed.
                 found = True
@@ -755,21 +696,28 @@ COMMIT
             else:
                 newrules.append(r)
 
-        # Add rule to the end if it was not already added.
-        if not found and not rule.remove:
-            newrules.append(rule)
+        if inserted:
+            if matches > 0:
+                rstr = _("Skipping inserting existing rule")
+                if rule.v6:
+                    rstr += " (v6)"
+                return rstr
+        else:
+            # Add rule to the end if it was not already added.
+            if not found and not rule.remove:
+                newrules.append(rule)
 
-        # Don't process non-existing or unchanged pre-exisiting rules
-        if not found and rule.remove and not self.dryrun:
-            rstr = _("Could not delete non-existent rule")
-            if rule.v6:
-                rstr += " (v6)"
-            return rstr
-        elif found and not rule.remove and not modified:
-            rstr = _("Skipping adding existing rule")
-            if rule.v6:
-                rstr += " (v6)"
-            return rstr
+            # Don't process non-existing or unchanged pre-exisiting rules
+            if not found and rule.remove and not self.dryrun:
+                rstr = _("Could not delete non-existent rule")
+                if rule.v6:
+                    rstr += " (v6)"
+                return rstr
+            elif found and not rule.remove and not modified:
+                rstr = _("Skipping adding existing rule")
+                if rule.v6:
+                    rstr += " (v6)"
+                return rstr
 
         if rule.v6:
             self.rules6 = newrules
@@ -792,8 +740,12 @@ COMMIT
         # Operate on the chains
         if self._is_enabled() and not self.dryrun:
             flag = ""
-            if modified or self._need_reload(rule.v6):
-                rstr = _("Rule updated")
+            if modified or self._need_reload(rule.v6) or inserted:
+                rstr = _("Rule ")
+                if inserted:
+                    rstr += _("inserted")
+                else:
+                    rstr += _("updated")
                 if rule.v6:
                     rstr += " (v6)"
                 if allow_reload:
@@ -826,24 +778,22 @@ COMMIT
                 if rc != 0:
                     raise UFWError(err_msg)
 
-                for s in self._get_rules_from_formatted(rule.format_rule(), \
-                                                        chain_prefix):
-                    debug([exe, flag, chain] + s.split())
-                    (rc, out) = cmd([exe, flag, chain] + s.split())
+                rule_str = "%s %s %s" % (flag, chain, rule.format_rule())
+                pat_log = re.compile(r'(-A +)(ufw6?-user-[a-z\-]+)(.*)')
+                for s in self._get_lists_from_formatted(rule_str, \
+                                                               chain_prefix):
+                    (rc, out) = cmd([exe] + s)
                     if rc != 0:
-                        print >> sys.stderr, out
+                        msg(out, sys.stderr)
                         UFWError(err_msg)
 
-                    # delete the RETURN rule then add it back, so it is at the
-                    # end
-                    if flag == "-A":
-                        (rc, out) = cmd([exe, '-D', chain, '-j', 'RETURN'])
+                    # delete any lingering RETURN rules (needed for upgrades)
+                    if flag == "-A" and pat_log.search(" ".join(s)):
+                        c = pat_log.sub(r'\2', " ".join(s))
+                        (rc, out) = cmd([exe, '-D', c, '-j', 'RETURN'])
                         if rc != 0:
-                            print >> sys.stderr, out
+                            debug("FAILOK: " + err_msg)
 
-                        (rc, out) = cmd([exe, '-A', chain, '-j', 'RETURN'])
-                        if rc != 0:
-                            print >> sys.stderr, out
         return rstr
 
     def get_app_rules_from_system(self, template, v6):
@@ -870,3 +820,132 @@ COMMIT
 
         return app_rules
 
+    def _chain_cmd(self, chain, args, fail_ok=False):
+        '''Perform command on chain'''
+        exe = "iptables"
+        if chain.startswith("ufw6"):
+            exe = "ip6tables"
+        (rc, out) = cmd([exe] + args)
+        if rc != 0:
+           err_msg = _("Could not perform '%s'") % (args)
+           if fail_ok:
+               debug("FAILOK: " + err_msg)
+           else: 
+               raise UFWError(err_msg)
+
+    def update_logging(self, level):
+        '''Update loglevel of running firewall'''
+        if not self._is_enabled():
+            return
+
+        if level not in self.loglevels.keys():
+            err_msg = _("Invalid log level '%s'") % (level)
+            raise UFWError(err_msg)
+
+        # make sure all the chains are here, it's redundant but helps make
+        # sure the chains are in a consistent state
+        err_msg = _("Could not update running firewall")
+        for c in self.chains['before'] + self.chains['user'] + \
+           self.chains['after'] + self.chains['misc']:
+            try:
+                self._chain_cmd(c, ['-L', c, '-n'])
+            except:
+                raise UFWError(err_msg)
+
+        # Flush all the logging chains except 'user'
+        try:
+            for c in self.chains['before'] + self.chains['after'] + \
+               self.chains['misc']:
+                self._chain_cmd(c, ['-F', c])
+                self._chain_cmd(c, ['-Z', c])
+        except:
+            raise UFWError(err_msg)
+
+        if level == "off":
+            # when off, insert a RETURN rule at the top of user rules, thus
+            # preserving the rules
+            for c in self.chains['user']:
+                self._chain_cmd(c, ['-D', c, '-j', 'RETURN'], fail_ok=True)
+                self._chain_cmd(c, ['-I', c, '-j', 'RETURN'])
+            return
+        else:
+            # when on, remove the RETURN rule at the top of user rules, thus
+            # honoring the log rules
+            for c in self.chains['user']:
+                self._chain_cmd(c, ['-D', c, '-j', 'RETURN'], fail_ok=True)
+
+        limit_args = ['-m', 'limit', '--limit', '3/min', '--limit-burst', '10']
+
+        # log levels of low and higher log blocked packets
+        if self.loglevels[level] >= self.loglevels["low"]:
+            # Setup the policy violation logging chains
+            largs = []
+            # log levels under high use limit
+            if self.loglevels[level] < self.loglevels["high"]:
+                largs = limit_args
+            for c in self.chains['after']:
+                for t in ['input', 'output', 'forward']:
+                    if c.endswith(t):
+                        if self.get_default_policy(t) == "reject" or \
+                           self.get_default_policy(t) == "deny":
+                            msg = "[UFW BLOCK] "
+                            try:
+                                self._chain_cmd(c, ['-A', c, '-j', 'LOG', \
+                                                    '--log-prefix', msg] + largs)
+                            except:
+                                raise
+                        elif self.loglevels[level] >= self.loglevels["medium"]:
+                            msg = "[UFW ALLOW] "
+                            try:
+                                self._chain_cmd(c, ['-A', c, '-j', 'LOG', \
+                                                    '--log-prefix', msg] + largs)
+                            except:
+                                raise
+
+            # Setup the miscellaneous logging chains
+            largs = []
+            # log levels under high use limit
+            if self.loglevels[level] < self.loglevels["high"]:
+                largs = limit_args
+
+            for c in self.chains['misc']:
+                if c.endswith("allow"):
+                    msg = "[UFW ALLOW] "
+                elif c.endswith("deny"):
+                    msg = "[UFW BLOCK] "
+                    if self.loglevels[level] >= self.loglevels["medium"]:
+                        # only log INVALID in medium and higher
+                        try:
+                            self._chain_cmd(c, ['-I', c, '-m', 'state', \
+                                                '--state', 'INVALID', \
+                                                '-j', 'RETURN'] + largs)
+                        except:
+                            raise
+                try:
+                    self._chain_cmd(c, ['-A', c, '-j', 'LOG', \
+                                        '--log-prefix', msg] + largs)
+                except:
+                    raise
+
+        # Setup the audit logging chains
+        if self.loglevels[level] >= self.loglevels["medium"]:
+            # loglevel full logs all packets without limit
+            largs = []
+
+            # loglevel high logs all packets with limit
+            if self.loglevels[level] < self.loglevels["full"]:
+                largs = limit_args
+
+            # loglevel medium logs all new packets with limit
+            if self.loglevels[level] < self.loglevels["high"]:
+                largs = ['-m', 'state', '--state', 'NEW'] + limit_args
+
+            msg = "[UFW AUDIT] "
+            for c in self.chains['before']:
+                try:
+                    self._chain_cmd(c, ['-I', c, '-j', 'LOG', \
+                                        '--log-prefix', msg] + largs)
+                except:
+                    raise UFWError(err_msg)
+
+
