@@ -1,7 +1,7 @@
 #
 # frontend.py: frontend interface for ufw
 #
-# Copyright (C) 2008 Canonical Ltd.
+# Copyright (C) 2008-2009 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -26,6 +26,17 @@ import ufw.util
 from ufw.util import error, warn
 from ufw.backend_iptables import UFWBackendIptables
 
+def allowed_command(cmd):
+    '''Return command if it is allowed, otherwise raise an exception'''
+    allowed_cmds = ['enable', 'disable', 'help', '--help', 'default', \
+                    'logging', 'status', 'version', '--version', 'allow', \
+                    'deny', 'reject', 'limit', 'reload', 'show' ]
+
+    if not cmd.lower() in allowed_cmds:
+        raise ValueError()
+
+    return cmd.lower()
+
 def parse_command(argv):
     '''Parse command. Returns tuple for action, rule, ip_version and dryrun.'''
     action = ""
@@ -36,6 +47,9 @@ def parse_command(argv):
     from_service = ""
     to_service = ""
     dryrun = False
+    insert_pos = ""
+    logtype = ""
+    loglevel = ""
 
     if len(argv) > 1 and argv[1].lower() == "--dry-run":
         dryrun = True
@@ -51,28 +65,58 @@ def parse_command(argv):
     if nargs < 2:
         raise ValueError()
 
-    allowed_cmds = ['enable', 'disable', 'help', '--help', 'default', \
-                    'logging', 'status', 'version', '--version', 'allow', \
-                    'deny', 'limit', 'reload' ]
-
-    if not argv[1].lower() in allowed_cmds:
-        raise ValueError()
-    else:
+    if argv[1].lower() in ['insert']:
         action = argv[1].lower()
+    else:
+        action = allowed_command(argv[1])
+
+    if action == "insert":
+        if nargs < 4:
+            raise ValueError()
+        insert_pos = argv[2]
+
+        # Using position '0' adds rule at end, which is potentially confusing
+        # for the end user
+        if insert_pos == "0":
+            err_msg = _("Cannot insert rule at position '%s'") % (insert_pos)
+            raise UFWError(err_msg)
+
+        # strip out 'insert NUM' and parse as normal
+        del argv[2]
+        del argv[1]
+        action = allowed_command(argv[1])
+        nargs = len(argv)
+
+        # error if use insert with non-rule commands
+        if action != "allow" and action != "deny" and action != "reject" and \
+           action != "limit":
+            raise ValueError()
 
     if action == "logging":
         if nargs < 3:
             raise ValueError()
         elif argv[2].lower() == "off":
             action = "logging-off"
-        elif argv[2].lower() == "on":
+        elif argv[2].lower() == "on" or argv[2].lower() == "low" or \
+             argv[2].lower() == "medium" or argv[2].lower() == "high" or \
+             argv[2].lower() == "full":
             action = "logging-on"
+            if argv[2].lower() != "on":
+                action += "_" + argv[2].lower()
         else:
             raise ValueError()
 
-    if action == "status":
-        if nargs > 2 and argv[2].lower() == "verbose":
+    if action == "status" and nargs > 2:
+        if argv[2].lower() == "verbose":
             action = "status-verbose"
+        elif argv[2].lower() == "numbered":
+            action = "status-numbered"
+
+    if action == "show":
+        if nargs == 2:
+            raise ValueError()
+        elif argv[2].lower() == "raw":
+            action = "show-raw"
 
     if action == "default":
         if nargs < 3:
@@ -81,16 +125,37 @@ def parse_command(argv):
             action = "default-deny"
         elif argv[2].lower() == "allow":
             action = "default-allow"
+        elif argv[2].lower() == "reject":
+            action = "default-reject"
         else:
             raise ValueError()
 
-    if action == "allow" or action == "deny" or action == "limit":
+    if action == "allow" or action == "deny" or action == "reject" or \
+       action == "limit":
+        if nargs > 2 and (argv[2].lower() == "log" or \
+                          argv[2].lower() == 'log-all'):
+            if nargs < 4:
+                raise ValueError()
+            logtype = argv[2].lower()
+
+            # strip out 'log' or 'log-all' and parse as normal
+            del argv[2]
+            nargs = len(argv)
+
         if nargs < 3 or nargs > 12:
             raise ValueError()
 
-        rule = ufw.common.UFWRule(action, "any", "any")
+        rule_action = action
+        if logtype != "":
+            rule_action += "_" + logtype
+        rule = ufw.common.UFWRule(rule_action, "any", "any")
         if remove:
             rule.remove = remove
+        elif insert_pos != "":
+            try:
+                rule.set_position(insert_pos)
+            except Exception:
+                raise
         if nargs == 3:
             # Short form where only app or port/proto is given
             if ufw.applications.valid_profile_name(argv[2]):
@@ -341,6 +406,8 @@ def parse_application_command(argv):
             action = "default-allow"
         elif argv[2].lower() == "deny":
             action = "default-deny"
+        elif argv[2].lower() == "reject":
+            action = "default-reject"
         elif argv[2].lower() == "skip":
             action = "default-skip"
         else:
@@ -355,20 +422,24 @@ def get_command_help():
 Usage: ''') + ufw.common.programName + _(''' COMMAND
 
 Commands:
-  enable			enables the firewall
-  disable			disables the firewall
-  default ARG			set default policy to ALLOW or DENY
-  logging ARG			set logging to ON or OFF
-  allow|deny RULE		allow or deny RULE
-  delete allow|deny RULE	delete the allow/deny RULE
-  status			show firewall status
-  version			display version information
+ enable				enables the firewall
+ disable			disables the firewall
+ default ARG			set default policy to ALLOW, DENY or REJECT
+ logging ARG			set logging to OFF, ON or LEVEL
+ allow|deny|reject ARG		add allow, deny or reject RULE
+ delete RULE		 	delete the RULE
+ insert NUM RULE	 	insert RULE at NUM
+ status 			show firewall status
+ status numbered		show firewall status as numbered list of RULES
+ show ARG			show firewall report
+ version			display version information
 
 Application profile commands:
-  app list			list application profiles
-  app info PROFILE		show information on PROFILE
-  app update PROFILE		update PROFILE
-  app default ARG		set profile policy to ALLOW, DENY or SKIP
+ app list			list application profiles
+ app info PROFILE		show information on PROFILE
+ app update PROFILE		update PROFILE
+ app default ARG		set profile policy to ALLOW, DENY, REJECT or
+				SKIP
 ''')
     return (msg)
 
@@ -384,6 +455,14 @@ class UFWFrontend:
         else:
             raise UFWError("Unsupported backend type '%s'" % (backend_type))
 
+        self._init_input_strings()
+
+    def _init_input_strings(self):
+        '''Initialize input strings for translations'''
+        self.no = _("n")
+        self.yes = _("y")
+        self.yes_full = _("yes")
+
     def set_enabled(self, enabled):
         '''Toggles ENABLED state in of <config_dir>/ufw/ufw.conf'''
         res = ""
@@ -425,7 +504,7 @@ class UFWFrontend:
                 # Report the error
                 error(error_str)
 
-            res = _("Firewall started and enabled on system startup")
+            res = _("Firewall is active and enabled on system startup")
         else:
             try:
                 self.backend.stop_firewall()
@@ -454,19 +533,24 @@ class UFWFrontend:
         res = ""
         try:
             res = self.backend.set_loglevel(level)
-            if self.backend._is_enabled():
-                # have to just restart because of ordering of LOG rules
-                self.backend.stop_firewall()
-                self.backend.start_firewall()
         except UFWError, e:
             error(e.value)
 
         return res
 
-    def get_status(self, verbose=False):
+    def get_status(self, verbose=False, show_count=False):
         '''Shows status of firewall'''
         try:
-            out = self.backend.get_status(verbose)
+            out = self.backend.get_status(verbose, show_count)
+        except UFWError, e:
+            error(e.value)
+
+        return out
+
+    def get_show_raw(self):
+        '''Shows raw output of firewall'''
+        try:
+            out = self.backend.get_running_raw()
         except UFWError, e:
             error(e.value)
 
@@ -520,30 +604,84 @@ class UFWFrontend:
                         r = tmp.dup_rule()
                         r.remove = rule.remove
                         r.set_action(rule.action)
+                        r.set_logtype(rule.logtype)
                         rules.append(r)
                 else:
                     rules = self.backend.get_app_rules_from_template(rule)
+                    # Reverse the order of rules for inserted rules, so they
+                    # are inserted in the right order
+                    if rule.position > 0:
+                        rules.reverse()
             except Exception:
                 raise
 
         count = 0
         set_error = False
+        pos_err_msg = _("Invalid position '")
+        num_v4 = self.backend.get_rules_count(False)
+        num_v6 = self.backend.get_rules_count(True)
         for i, r in enumerate(rules):
             count = i
+            if r.position > num_v4 + num_v6:
+                pos_err_msg += str(r.position) + "'"
+                raise UFWError(pos_err_msg)
             try:
                 if self.backend.use_ipv6():
                     if ip_version == "v4":
+                        if r.position > num_v4:
+                            pos_err_msg += str(r.position) + "'"
+                            raise UFWError(pos_err_msg)
                         r.set_v6(False)
                         tmp = self.backend.set_rule(r)
                     elif ip_version == "v6":
+                        if r.position > num_v4:
+                            r.set_position(r.position - num_v4)
+                        elif r.position != 0 and r.position <= num_v4:
+                            pos_err_msg += str(r.position) + "'"
+                            raise UFWError(pos_err_msg)
                         r.set_v6(True)
                         tmp = self.backend.set_rule(r)
                     elif ip_version == "both":
+                        user_pos = r.position # user specified position
                         r.set_v6(False)
+                        if not r.remove and user_pos > num_v4:
+			    # The user specified a v6 rule, so try to find a
+			    # match in the v4 rules and use its position.
+                            p = self.backend.find_other_position(user_pos - \
+                                                                 num_v4, True)
+                            if p > 0:
+                                r.set_position(p)
+                            else:
+                                # If not found, then add the rule
+                                r.set_position(0)
                         tmp = self.backend.set_rule(r)
+
+                        # We need to readjust the position since the number
+                        # the number of ipv4 rules increased
+                        if not r.remove and user_pos > 0:
+                            num_v4 = self.backend.get_rules_count(False)
+                            r.set_position(user_pos + 1)
+
                         r.set_v6(True)
+                        if not r.remove and r.position > 0 and \
+                           r.position <= num_v4:
+			    # The user specified a v4 rule, so try to find a
+			    # match in the v6 rules and use its position.
+                            p = self.backend.find_other_position(r.position, \
+                                                                 False)
+                            if p > 0:
+                                # Subtract count since the list is reversed
+                                r.set_position(p - count)
+                            else:
+                                # If not found, then add the rule
+                                r.set_position(0)
                         if tmp != "":
                             tmp += "\n"
+
+                        # Readjust position to send to set_rule
+                        if not r.remove and r.position > num_v4:
+                            r.set_position(r.position - num_v4)
+
                         tmp += self.backend.set_rule(r)
                     else:
                         err_msg = _("Invalid IP version '%s'") % (ip_version)
@@ -607,18 +745,28 @@ class UFWFrontend:
            based on return values from parse_command().
         '''
         res = ""
-        if action == "logging-on":
-            res = self.set_loglevel("on")
+        if action.startswith("logging-on"):
+            tmp = action.split('_')
+            if len(tmp) > 1:
+                res = self.set_loglevel(tmp[1])
+            else:
+                res = self.set_loglevel("on")
         elif action == "logging-off":
             res = self.set_loglevel("off")
         elif action == "default-allow":
             res = self.set_default_policy("allow")
         elif action == "default-deny":
             res = self.set_default_policy("deny")
+        elif action == "default-reject":
+            res = self.set_default_policy("reject")
         elif action == "status":
             res = self.get_status()
         elif action == "status-verbose":
             res = self.get_status(True)
+        elif action == "show-raw":
+            res = self.get_show_raw()
+        elif action == "status-numbered":
+            res = self.get_status(False, True)
         elif action == "enable":
             res = self.set_enabled(True)
         elif action == "disable":
@@ -630,7 +778,22 @@ class UFWFrontend:
                 res = _("Firewall reloaded")
             else:
                 res = _("Firewall not enabled (skipping reload)")
-        elif action == "allow" or action == "deny" or action == "limit":
+        elif action == "allow" or action == "deny" or action == "reject" or \
+             action == "limit":
+            # allow case insensitive matches for application rules
+            try:
+                if rule.dapp != "":
+                    tmp = self.backend.find_application_name(rule.dapp)
+                    if tmp != rule.dapp:
+                        rule.dapp = tmp
+                        rule.set_port(tmp, "dst")
+                if rule.sapp != "":
+                    tmp = self.backend.find_application_name(rule.sapp)
+                    if tmp != rule.sapp:
+                        rule.sapp = tmp
+                        rule.set_port(tmp, "src")
+            except UFWError, e:
+                error(e.value)
             res = self.set_rule(rule, ip_version)
         else:
             err_msg = _("Unsupported action '%s'") % (action)
@@ -758,6 +921,8 @@ class UFWFrontend:
             policy = "allow"
         elif default == "drop":
             policy = "deny"
+        elif default == "reject":
+            policy = "reject"
         else:
             err_msg = _("Unknown policy '%s'") % (default)
             raise UFWError(err_msg)
@@ -785,6 +950,8 @@ class UFWFrontend:
             res = self.set_default_application_policy("allow")
         elif action == "default-deny":
             res = self.set_default_application_policy("deny")
+        elif action == "default-reject":
+            res = self.set_default_application_policy("reject")
         elif action == "default-skip":
             res = self.set_default_application_policy("skip")
         elif action == "list":
@@ -792,9 +959,14 @@ class UFWFrontend:
         elif action == "info":
             res = self.get_application_info(profile)
         elif action == "update" or action == "update-with-new":
-            res = self.application_update(profile)
+            str1 = self.application_update(profile)
+            str2 = ""
             if action == "update-with-new":
-                res += "\n" + self.application_add(profile)
+                str2 = self.application_add(profile)
+
+            if str1 != "" and str2 != "":
+                str1 += "\n"
+            res = str1 + str2
         else:
             err_msg = _("Unsupported action '%s'") % (action)
             raise UFWError(err_msg)
@@ -806,10 +978,11 @@ class UFWFrontend:
         proceed = True
         if self.backend.do_checks and ufw.util.under_ssh():
             prompt = _("Command may disrupt existing ssh connections.")
-            prompt += _(" Proceed with operation (y|n)? ")
+            prompt += _(" Proceed with operation (%s|%s)? ") % \
+                       (self.yes, self.no)
             os.write(sys.stdout.fileno(), prompt)
             ans = sys.stdin.readline().lower().strip()
-            if ans != "y" and ans != "yes":
+            if ans != "y" and ans != self.yes and ans != self.yes_full:
                 proceed = False
 
         return proceed
