@@ -1,7 +1,7 @@
 #
 # util.py: utility functions for ufw
 #
-# Copyright (C) 2008 Canonical Ltd.
+# Copyright (C) 2008-2009 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -16,6 +16,7 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+import errno
 import os
 import re
 import shutil
@@ -263,20 +264,38 @@ def cmd_pipe(command1, command2):
 
 def error(msg, exit=True):
     '''Print error message and exit'''
-    print >> sys.stderr, "ERROR: %s" % (msg)
+    try:
+        print >> sys.stderr, "ERROR: %s" % (msg)
+    except IOError:
+        pass
+
     if exit:
         sys.exit(1)
 
 
 def warn(msg):
     '''Print warning message'''
-    print >> sys.stderr, "WARN: %s" % (msg)
+    try:
+        print >> sys.stderr, "WARN: %s" % (msg)
+    except IOError:
+        pass
+
+
+def msg(msg, output=sys.stdout):
+    '''Print message'''
+    try:
+        print >> output, "%s" % (msg)
+    except IOError:
+        pass
 
 
 def debug(msg):
     '''Print debug message'''
     if debugging:
-        print >> sys.stderr, "DEBUG: %s" % (msg)
+        try:
+            print >> sys.stderr, "DEBUG: %s" % (msg)
+        except IOError:
+            pass
 
 
 def word_wrap(text, width):
@@ -495,3 +514,12 @@ def _address4_to_network(addr):
 
     return network + "/" + orig_nm
 
+def get_iptables_version():
+    '''Return iptables version'''
+    exe = 'iptables'
+    (rc, out) = cmd([exe, '-V'])
+    if rc != 0:
+        raise OSError(errno.ENOENT, "Error running '%s'" % (exe))
+    tmp = re.split('\s', out)
+    return re.sub('^v', '', tmp[1])
+
