@@ -1,6 +1,6 @@
 #!/bin/bash
 
-#    Copyright (C) 2008 Canonical Ltd.
+#    Copyright (C) 2008-2009 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -207,4 +207,40 @@ for i in allow deny limit; do
     done
 done
 
+echo "Man page (reject)" >> $TESTTMP/result
+do_cmd "0" --dry-run reject auth
+
+echo "Reject" >> $TESTTMP/result
+from="192.168.0.1"
+to="10.0.0.1"
+do_cmd "0" --dry-run reject to any port auth from any port smtp
+do_cmd "0" --dry-run delete reject to any port auth from any port smtp
+do_cmd "0" --dry-run reject to $to port domain from $from port auth
+do_cmd "0" --dry-run delete reject to $to port domain from $from port auth
+for i in any tcp udp ; do
+    p="/$i"
+    if [ "$i" = "any" ]; then
+        p=""
+    else
+        do_cmd "0" --dry-run reject 23,21,15:19,22$p
+    fi
+    do_cmd "0" --dry-run reject 116$p
+done
+
+echo "Insert" >> $TESTTMP/result
+do_cmd "0" null allow 22
+do_cmd "0" null allow 23
+
+do_cmd "0" null insert 1 allow 9999
+do_cmd "0" null insert 1 allow log 9998
+do_cmd "0" null insert 2 reject to 192.168.0.1 from 10.0.0.1
+cat $TESTPATH/var/lib/ufw/user.rules >> $TESTTMP/result
+
+do_cmd "0" null delete allow 22
+do_cmd "0" null delete allow 23
+do_cmd "0" null delete allow 9999
+do_cmd "0" null delete allow log 9998
+do_cmd "0" null delete reject to 192.168.0.1 from 10.0.0.1
+cat $TESTPATH/var/lib/ufw/user.rules >> $TESTTMP/result
+
 exit 0
