@@ -27,31 +27,37 @@ do_cmd "1" null --dry-run default
 do_cmd "1" null --dry-run default foo
 do_cmd "1" null --dry-run default accept
 do_cmd "1" null --dry-run defaul allow
+do_cmd "1" null --dry-run default limit
 
 echo "TESTING ARGS (enable/disable)" >> $TESTTMP/result
 # bad
 do_cmd "1" null --dry-run enabled
 do_cmd "1" null --dry-run disabled
 
-echo "TESTING ARGS (allow/deny)" >> $TESTTMP/result
+echo "TESTING ARGS (allow/deny/limit)" >> $TESTTMP/result
 do_cmd "1" null --dry-run allow
 do_cmd "1" null --dry-run deny
+do_cmd "1" null --dry-run limit
 
-echo "TESTING ARGS (allow/deny bad port)" >> $TESTTMP/result
+echo "TESTING ARGS (allow/deny/limit bad port)" >> $TESTTMP/result
 do_cmd "1" null --dry-run alow 25
 do_cmd "1" null --dry-run dny 25
+do_cmd "1" null --dry-run limt 25
 do_cmd "1" null --dry-run allow 25a
 do_cmd "1" null --dry-run deny 25a
+do_cmd "1" null --dry-run limit 25a
 do_cmd "1" null --dry-run allow 65536
 do_cmd "1" null --dry-run deny 65536
+do_cmd "1" null --dry-run limit 65536
 do_cmd "1" null --dry-run allow 0
 do_cmd "1" null --dry-run deny 0
+do_cmd "1" null --dry-run limit 0
 do_cmd "1" null --dry-run deny XXX
 do_cmd "1" null --dry-run deny foobar
 
-echo "TESTING ARGS (allow/deny bad to/from)" >> $TESTTMP/result
+echo "TESTING ARGS (allow/deny/limit bad to/from)" >> $TESTTMP/result
 ip="2001:db8:3:4:5:6:7:8"
-for action in allow deny
+for action in allow deny limit
 do
         do_cmd "1" null --dry-run $action prot tcp from any
         do_cmd "1" null --dry-run $action proto tcp fro any
@@ -93,7 +99,7 @@ do
 	do_cmd "1" null --dry-run $action to to $ip port smtp
 done
 
-echo "TESTING ARGS (allow/deny bad ip)" >> $TESTTMP/result
+echo "TESTING ARGS (bad ip)" >> $TESTTMP/result
 do_cmd "1" null --dry-run allow to 2001:db8:::/32
 do_cmd "1" null --dry-run allow to 2001:db8::/129
 do_cmd "1" null --dry-run allow to 2001:gb8::/32
@@ -112,14 +118,16 @@ do_cmd "1" null --dry-run allow to 2001:0db8:0000:0000:0000:0000:0000:0000/129
 do_cmd "1" null --dry-run allow to 2001:0db8:0000:0000:0000:0000:0000:00000/128
 do_cmd "1" null --dry-run allow to 2001:0db8:0000:0000:0000:0000:0000:00000/12a
 
-echo "TESTING ARGS (delete allow/deny)" >> $TESTTMP/result
+echo "TESTING ARGS (delete)" >> $TESTTMP/result
 do_cmd "1" null --dry-run delete
 
-echo "TESTING ARGS (allow/deny mixed ipv4/ipv6)" >> $TESTTMP/result
+echo "TESTING ARGS (allow/deny/limit mixed ipv4/ipv6)" >> $TESTTMP/result
 do_cmd "1" null --dry-run allow to 10.0.0.1 from 2001:db8::/32
 do_cmd "1" null --dry-run deny to 10.0.0.1 port 25 from 2001:db8::/32 proto tcp
+do_cmd "1" null --dry-run limit to 10.0.0.1 port 25 from 2001:db8::/32 proto tcp
 do_cmd "1" null --dry-run allow to 2001:db8::/32 port 25 from 10.0.0.1 proto udp
 do_cmd "1" null --dry-run deny to 2001:db8::/32 from 10.0.0.1
+do_cmd "1" null --dry-run limit to 2001:db8::/32 from 10.0.0.1
 
 echo "TESTING BAD SERVICES" >> $TESTTMP/result
 do_cmd "1" null --dry-run allow smtp/udp
@@ -133,4 +141,35 @@ do_cmd "1" null --dry-run allow to any port 23 from any port tftp proto tcp
 do_cmd "1" null --dry-run allow to any port smtp from any port ssh proto udp
 do_cmd "1" null --dry-run allow to any port tftp from any port ssh proto tcp
 
+echo "TESTING BAD MULTIPORTS" >> $TESTTMP/result
+for i in allow deny limit; do
+    for j in from to; do
+        do_cmd "1" null --dry-run $i $j any port 20,21
+        do_cmd "1" null --dry-run $i $j any port 20,2L proto udp
+        do_cmd "1" null --dry-run $i $j any port 2o,21 proto tcp
+        do_cmd "1" null --dry-run $i $j any port 20, proto udp
+        do_cmd "1" null --dry-run $i $j any port ,20 proto tcp
+        do_cmd "1" null --dry-run $i $j any port ,20, proto udp
+        do_cmd "1" null --dry-run $i $j any port 20: proto tcp
+        do_cmd "1" null --dry-run $i $j any port :20 proto udp
+        do_cmd "1" null --dry-run $i $j any port :20: proto tcp
+        do_cmd "1" null --dry-run $i $j any port 20:65536 proto udp
+        do_cmd "1" null --dry-run $i $j any port 0:65 proto tcp
+        do_cmd "1" null --dry-run $i $j any port ,20:24 proto udp
+        do_cmd "1" null --dry-run $i $j any port 20:24, proto tcp
+        do_cmd "1" null --dry-run $i $j any port ,20:24, proto udp
+        do_cmd "1" null --dry-run $i $j any port 24:20 proto tcp
+        do_cmd "1" null --dry-run $i $j any port 2A:20 proto tcp
+        do_cmd "1" null --dry-run $i $j any port 24:2o proto tcp
+        do_cmd "1" null --dry-run $i $j any port http,smtp proto tcp
+        do_cmd "1" null --dry-run $i $j any port 80,smtp proto tcp
+        do_cmd "1" null --dry-run $i $j any port http,25 proto tcp
+    done
+
+    do_cmd "1" null --dry-run $i to any port 20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35 from any port 20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35 proto tcp
+    do_cmd "1" null --dry-run $i to any port 20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35 from any port 20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35 proto udp
+    do_cmd "1" null --dry-run $i from any port 20,21,22,23,24,25,26,27,28,29,30,31,32,33,34:39 to any port 20,21,22,23,24,25,26,27,28,29,30,31,32,33,34:39 proto tcp
+    do_cmd "1" null --dry-run $i from any port 20,21,22,23,24,25,26,27,28,29,30,31,32,33,34:39 to any port 20,21,22,23,24,25,26,27,28,29,30,31,32,33,34:39 proto udp
+done
+
 exit 0
