@@ -1,6 +1,6 @@
 #!/bin/bash
 
-#    Copyright (C) 2008 Canonical Ltd.
+#    Copyright (C) 2008-2009 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -79,7 +79,7 @@ grep -A2 "tuple" $TESTPATH/var/lib/ufw/user6.rules >> $TESTTMP/result
 echo "TO/FROM" >> $TESTTMP/result
 from="2001:db8::/32"
 to="2001:db8:3:4:5:6:7:8"
-for x in allow deny limit
+for x in allow deny limit reject
 do
         context="2"
         if [ "$x" = "limit" ]; then
@@ -321,5 +321,5 @@ do_cmd "0" delete allow 23,21,15:19,22/udp
 grep -A2 "tuple" $TESTPATH/var/lib/ufw/user.rules >> $TESTTMP/result
 grep -A2 "tuple" $TESTPATH/var/lib/ufw/user6.rules >> $TESTTMP/result
 
-
+cleanup
 exit 0
