@@ -1,6 +1,6 @@
 #!/bin/bash
 
-#    Copyright (C) 2008 Canonical Ltd.
+#    Copyright (C) 2008-2009 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -46,3 +46,8 @@ do_cmd() {
         echo $individual > $statsdir/individual
 }
 
+cleanup() {
+    do_cmd "0" disable
+    $TESTPATH/usr/share/ufw/ufw-init flush-all
+}
+
